==> mxv-v1.06-source.tar.gz: localdefs.h <==
// localdefs.h

/******************************************************************************
 *
 *  MiXViews - an X window system based sound & data editor/processor
 *
 *  Copyright (c) 1993, 1994 Regents of the University of California
 *
 *  Author:     Douglas Scott
 *  Date:       December 13, 1994
 *
 *  Permission to use, copy and modify this software and its documentation
 *  for research and/or educational purposes and without fee is hereby granted,
 *  provided that the above copyright notice appear in all copies and that
 *  both that copyright notice and this permission notice appear in
 *  supporting documentation. The author reserves the right to distribute this
 *  software and its documentation.  The University of California and the author
 *  make no representations about the suitability of this software for any 
 *  purpose, and in no event shall University of California be liable for any
 *  damage, loss of data, or profits resulting from its use.
 *  It is provided "as is" without express or implied warranty.
 *
 ******************************************************************************/


#ifndef LOCALDEFS_H
#define LOCALDEFS_H

// prototypes for bzero(), etc.

#include <X11/Xfuncs.h>

#ifdef __cplusplus

#ifndef NeXT
extern "C" {
#ifndef sgi
	extern int   gettimeofday(struct timeval *, struct timezone *);
#endif
	extern void cfree(void *);
	extern char *index(const char *, int); 
	extern char *rindex(const char *, int);
}
#endif

// here is the include for builtin...
#include <builtin.h>

#endif /* __cplusplus */

#ifndef __GNUC__
#include <math.h>
#include <string.h>
#endif

#include <stdio.h>
#include <stdlib.h>

typedef char *addr;
extern char* kFormat(double, double);
extern char* iFormat(double, double);

#ifdef __cplusplus

#include <InterViews/defs.h>

typedef double (*process_fun)(double, double);
typedef boolean (*CharCheckFun)(const char);

inline double roundUpToNearest(double val, double nearest) {
	double inc = (val>0.0) ? 0.9999999999999 : 0;
	return (int) ((val/nearest) + inc) * nearest;
}
inline double roundDownToNearest(double val, double nearest) {
	return (int) (val/nearest) * nearest;
}

inline int roundDown(double val) {
	return (val >= 0.0) ? int(val) : -int(-val + 0.99999999999999);
}

inline int roundUp(double val) {
	return (val >= 0.0) ? int(val + 0.99999999999999) : -int(-val);
}

inline double lowerPowerOfTen(double val) {
	return pow(double(10), double(roundDown(log10(val))));
}

inline double realMax(double x, double y) {
	return	(x <= 0.0) ? ((y >= 0.0) ? (x + y < 0.0) ? x : y : min(x, y))
		: ((y >= 0.0) ? max(x, y) : ((x + y < 0.0) ? y : x));
}

inline int realMax(int x, int y) {
	return	(x <= 0) ? ((y >= 0) ? (x + y < 0) ? x : y : min(x, y))
		: ((y >= 0) ? max(x, y) : ((x + y < 0) ? y : x));
}

// this converts any enum whose elements follow the linear series
// { 0, 1, 2, 3, ... } into the exponential series { 0, 1, 2, 4, ... }
// for use in Choice items which require the latter

inline int linearEnumToPowerOfTwo(int linEnum) {
	return (linEnum <= 0) ? 0 : 1 << (linEnum - 1);
}

// convert the other way

inline int powerOfTwoToLinearEnum(int power) {
	double logPow = log(double(power));
	return (power <= 0) ? 0 : int(logPow/double(log(2.0))) + 1;
}

inline const char * pstring(char *str, const char *msg) {
	sprintf(str, "%s", msg);
	return str;
}

inline const char * pstring(char *str, int i) {
	sprintf(str, "%d", i);
	return str;
}

inline const char * pstring(char *str, double d) {
	sprintf(str, "%f", d);
	return str;
}

inline char *newstr(const char *str) {
	return str == NULL ? (char *) str : strcpy( new char[strlen( str ) + 1], str );
}

class Debugger {
private:
	static int depth;
	char *function;
	int indent() {
		for(int i=0; i < depth; i++) fprintf(stderr, "\t"); return 0;
	}
public:
	Debugger(const char *fun) {
		function = new char[strlen(fun) + 1];
		strcpy(function, fun);
		indent();
		fprintf(stderr,"%s%s\n", "--> ", function);
		depth++;
	}
	~Debugger() {
		depth--;
		indent();
		fprintf(stderr, "%s%s\n", "<-- ", function);
		delete function;
	}
};

#ifdef debug
#define BUG(function) Debugger d(function);
#else
#define BUG(function)
#endif	/* debug */

enum DataType {
	NoData=0, CharData, MuLawData, ShortData, FloatData, DoubleData
};

enum FileType {
	Generic_Data=0, Sound_Data, LPC_Data, PCH_Data,
	FFT_Data, EVP_Data, Pvoc_Data
};

enum RangeUnit {
	ChannelUnit, ChannelSpecialUnit, FrameUnit, FrameTimeUnit, FrameSmpteUnit
};

enum Response {
	Cancel = -1, NullResponse = 0, Yes = 1, No = 2
};

enum EnvelopeType {
	AbsoluteMagnitude, PeakToPeakAmplitude, RMSAmplitude, Decibels
};

enum RunStatus { Error = -1, Stop = 0, Run = 1 };

#define redefined

#endif /* __cplusplus */

/* I use libg++'s String class rather than the one in InterViews */

#ifdef String
#undef String
#endif

/* the follwing is used for memory debugging */

#ifdef sparc
#include <malloc.h>
#endif

#ifdef PRINT_DTOR
#define DPRINT(loc) fprintf(stderr, "deleting " #loc "(this == %x).\n", this)
#else
#define DPRINT(loc)
#endif

#ifdef PRINT_CTOR
#define CPRINT(loc) fprintf(stderr, "creating " #loc "(this == %x).\n", this)
#else
#define CPRINT(loc)
#endif

#endif /* LOCALDEFS_H */

==> mxv-v1.10-source.tar.gz: localdefs.h <==
// localdefs.h

/******************************************************************************
 *
 *  MiXViews - an X window system based sound & data editor/processor
 *
 *  Copyright (c) 1993, 1994 Regents of the University of California
 *
 *  Author:     Douglas Scott
 *  Date:       December 13, 1994
 *
 *  Permission to use, copy and modify this software and its documentation
 *  for research and/or educational purposes and without fee is hereby granted,
 *  provided that the above copyright notice appear in all copies and that
 *  both that copyright notice and this permission notice appear in
 *  supporting documentation. The author reserves the right to distribute this
 *  software and its documentation.  The University of California and the author
 *  make no representations about the suitability of this software for any 
 *  purpose, and in no event shall University of California be liable for any
 *  damage, loss of data, or profits resulting from its use.
 *  It is provided "as is" without express or implied warranty.
 *
 ******************************************************************************/


#ifndef LOCALDEFS_H
#define LOCALDEFS_H

// prototypes for bzero(), etc.

#include <X11/Xfuncs.h>

#include "mxv_types.h"

#ifdef __cplusplus

#ifndef NeXT
extern "C" {
#ifndef sgi
	extern int   gettimeofday(struct timeval *, struct timezone *);
#endif
	extern void cfree(void *);
	extern char *index(const char *, int); 
	extern char *rindex(const char *, int);
}
#endif

// here is the include for builtin...
#include <builtin.h>

#endif /* __cplusplus */

#ifndef __GNUC__
#include <math.h>
#include <string.h>
#endif

#include <stdio.h>
#include <stdlib.h>

extern char* kFormat(double, double);
extern char* iFormat(double, double);

#ifdef __cplusplus

inline double roundUpToNearest(double val, double nearest) {
	double inc = (val>0.0) ? 0.9999999999999 : 0;
	return (int) ((val/nearest) + inc) * nearest;
}
inline double roundDownToNearest(double val, double nearest) {
	return (int) (val/nearest) * nearest;
}

inline int roundDown(double val) {
	return (val >= 0.0) ? int(val) : -int(-val + 0.99999999999999);
}

inline int roundUp(double val) {
	return (val >= 0.0) ? int(val + 0.99999999999999) : -int(-val);
}

inline double lowerPowerOfTen(double val) {
	return pow(double(10), double(roundDown(log10(val))));
}

inline double realMax(double x, double y) {
	return	(x <= 0.0) ? ((y >= 0.0) ? (x + y < 0.0) ? x : y : min(x, y))
		: ((y >= 0.0) ? max(x, y) : ((x + y < 0.0) ? y : x));
}

inline int realMax(int x, int y) {
	return	(x <= 0) ? ((y >= 0) ? (x + y < 0) ? x : y : min(x, y))
		: ((y >= 0) ? max(x, y) : ((x + y < 0) ? y : x));
}

// this converts any enum whose elements follow the linear series
// { 0, 1, 2, 3, ... } into the exponential series { 0, 1, 2, 4, ... }
// for use in Choice items which require the latter

inline int linearEnumToPowerOfTwo(int linEnum) {
	return (linEnum <= 0) ? 0 : 1 << (linEnum - 1);
}

// convert the other way

inline int powerOfTwoToLinearEnum(int power) {
	double logPow = log(double(power));
	return (power <= 0) ? 0 : round(logPow/double(log(2.0))) + 1;
}

// inlines for converting ints, doubles, etc. to strings and back

inline const char * toString(char *str, const char *msg) {
	sprintf(str, "%s", msg);
	return str;
}

inline const char * toString(char *str, short s) {
        sprintf(str, "%d", s);
        return str;
}

inline const char * toString(char *str, int i) {
        sprintf(str, "%d", i);
        return str;
}

inline const char * toString(char *str, unsigned u) {
        sprintf(str, "%u", u);
        return str;
}

inline const char * toString(char *str, float f) {
        sprintf(str, "%f", f);
        return str;
}

inline const char * toString(char *str, double d) {
        sprintf(str, "%f", d);
        return str;
}

inline const char* fromString(const char* str,  const char*) {
    return str;
}

inline int fromString(const char* str,  int) {
    return atoi(str);
}

inline short fromString(const char* str,  short) {
    return atoi(str);
}

inline unsigned fromString(const char* str,  unsigned) {
    return unsigned(atoi(str));
}

inline float fromString(const char* str,  float) {
    return atof(str);
}

inline double fromString(const char* str,  double) {
    return atof(str);
}

// string duplicating

inline char *newstr(const char *str) {
	return str == NULL ? (char *) str : strcpy( new char[strlen( str ) + 1], str );
}

class Debugger {
private:
	static int depth;
	char *function;
	int indent() {
		for(int i=0; i < depth; i++) fprintf(stderr, "\t"); return 0;
	}
public:
	Debugger(const char *fun) {
		function = new char[strlen(fun) + 1];
		strcpy(function, fun);
		indent();
		fprintf(stderr,"%s%s\n", "--> ", function);
		depth++;
	}
	~Debugger() {
		depth--;
		indent();
		fprintf(stderr, "%s%s\n", "<-- ", function);
		delete function;
	}
};

#ifdef debug
#define BUG(function) Debugger d(function);
#else
#define BUG(function)
#endif	/* debug */

#endif /* __cplusplus */

/* the following is used for memory debugging */

#ifdef sparc
#include <malloc.h>
#endif

#ifdef PRINT_DTOR
#define DPRINT(loc) fprintf(stderr, "deleting " #loc "(this == %x).\n", this)
#else
#define DPRINT(loc)
#endif

#ifdef PRINT_CTOR
#define CPRINT(loc) fprintf(stderr, "creating " #loc "(this == %x).\n", this)
#else
#define CPRINT(loc)
#endif

#endif /* LOCALDEFS_H */
